@@ -68,16 +68,32 @@ def main():
 		       "test", "port", "separator", "no_login", "no_password",
 		       "snmp_version", "community", "snmp_auth_prot", "snmp_sec_level",
 		       "snmp_priv_prot", "snmp_priv_passwd", "snmp_priv_passwd_script",
-		       "udpport","inet4_only","inet6_only"]
+		       "udpport","inet4_only","inet6_only",
+		       "power_timeout", "shell_timeout", "login_timeout", "power_wait" ]
 
 	atexit.register(atexit_handler)
 
+	snmp_define_defaults ()
+
 	options=check_input(device_opt,process_input(device_opt))
 
-	show_docs(options)
+	docs = { }
+	docs["shortdesc"] = "Fence agent for Intel Modular"
+	docs["longdesc"] = "fence_intelmodular is an I/O Fencing agent \
+which can be used with Intel Modular device (tested on Intel MFSYS25, should \
+work with MFSYS35 as well). \
+\n.P\n\
+Note: Since firmware update version 2.7, SNMP v2 write support is \
+removed, and replaced by SNMP v3 support. So agent now has default \
+SNMP version 3. If you are using older firmware, please supply -d \
+for command line and snmp_version option for your cluster.conf."
+	docs["vendorurl"] = "http://www.intel.com"
+	show_docs(options, docs)
 
 	# Operate the fencing device
-	fence_action(FencingSnmp(options), options, set_power_status, get_power_status, get_outlets_status)
+	result = fence_action(FencingSnmp(options), options, set_power_status, get_power_status, get_outlets_status)
+	
+	sys.exit(result)
 
 if __name__ == "__main__":
 	main()
