@@ -9,6 +9,9 @@
  *  http://ipmitool.sourceforge.net
  *
  */
+
+#include "clusterautoconfig.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -26,8 +29,10 @@
 #define ST_POWEROFF 2
 #define ST_GENERIC_RESET 3
 #define ST_CYCLE 4
+#define ST_DIAG 5
 
-#define DEFAULT_TIMEOUT 10
+#define DEFAULT_TIMEOUT 20
+#define DEFAULT_POWER_WAIT 2
 
 #define DEFAULT_METHOD "onoff"
 
@@ -78,6 +83,7 @@ struct ipmi {
 	int i_verbose;
 	int i_lanplus;
 	int i_timeout;
+	int i_power_wait;
 	int i_cipher;
 };
 
@@ -162,10 +168,12 @@ struct xml_parameter_s xml_parameters[]={
   {"passwd_script","-S",0,"string",NULL,"Script to retrieve password (if required)"},
   {"lanplus","-P",0,"boolean",NULL,"Use Lanplus"},
   {"login","-l",0,"string",NULL,"Username/Login (if required) to control power on IPMI device"},
-  {"action","-o",0,"string","reboot","Operation to perform. Valid operations: on, off, reboot, status, list, monitor or metadata"},
+  {"action","-o",0,"string","reboot","Operation to perform. Valid operations: on, off, reboot, status, list, diag, monitor or metadata"},
   {"timeout","-t",0,"string",NULL,"Timeout (sec) for IPMI operation"},
   {"cipher","-C",0,"string",NULL,"Ciphersuite to use (same as ipmitool -C parameter)"},
   {"method","-M",0,"string",DEFAULT_METHOD,"Method to fence (onoff or cycle)"},
+  {"power_wait","-T",0,"string","2","Wait X seconds after on/off operation"},
+  {"delay","-f",0,"string",NULL,"Wait X seconds before fencing is started"},
   {"verbose","-v",0,"boolean",NULL,"Verbose mode"}};
 
 /*
@@ -299,6 +307,10 @@ build_cmd(char *command, size_t cmdlen, struct ipmi *ipmi, int op)
 		snprintf(arg, sizeof(arg),
 			 "%s chassis power cycle", cmd);
 		break;
+	case ST_DIAG:
+		snprintf(arg, sizeof(arg),
+			 "%s chassis power diag", cmd);
+		break;
 	}
 
 	strncpy(command, arg, cmdlen);
@@ -485,7 +497,7 @@ ipmi_on(struct ipmi *ipmi)
 		if (ret != 0)
 			return ret;
 
-		sleep(2);
+		sleep(ipmi->i_power_wait);
 		--retries;
 		ret = ipmi_op(ipmi, ST_STATUS, power_status);
 
@@ -516,6 +528,16 @@ ipmi_cycle(struct ipmi *ipmi)
 	return ret;
 }
 
+static int
+ipmi_diag(struct ipmi *ipmi)
+{
+	int ret;
+	
+	ret = ipmi_op(ipmi, ST_DIAG, power_off_complete);
+	
+	return ret;
+}
+
 
 /**
   Squash all our private data
@@ -548,6 +570,7 @@ ipmi_destroy(struct ipmi *i)
 static struct ipmi *
 ipmi_init(struct ipmi *i, char *host, char *authtype,
 	  char *user, char *password, int lanplus, int verbose,int timeout,
+	  int power_wait,
 	  int cipher)
 {
 	const char *p;
@@ -620,6 +643,7 @@ ipmi_init(struct ipmi *i, char *host, char *authtype,
 	i->i_verbose = verbose;
 	i->i_lanplus = lanplus;
 	i->i_timeout = timeout;
+	i->i_power_wait = power_wait;
 	i->i_cipher = cipher;
 
 	return i;
@@ -684,7 +708,9 @@ get_options_stdin(char *ip, size_t iplen,
 		  char *user, size_t userlen,
 		  char *op, size_t oplen,
 		  int *lanplus, int *verbose,int *timeout,
-	          int *cipher, char *method, int methodlen)
+		  int *power_wait,
+	          int *cipher, char *method, int methodlen,
+	          char *delay, size_t delaylen)
 {
 	char in[256];
 	int line = 0;
@@ -751,6 +777,10 @@ get_options_stdin(char *ip, size_t iplen,
 			if ((sscanf(val,"%d",timeout)!=1) || *timeout<1) {
 			    *timeout=DEFAULT_TIMEOUT;
 			}
+		} else if (!strcasecmp(name,"power_wait")) {
+			if ((sscanf(val,"%d",power_wait)!=1) || *power_wait<1) {
+			    *power_wait=DEFAULT_POWER_WAIT;
+			}
 		} else if (!strcasecmp(name,"cipher")) {
 			if ((sscanf(val,"%d",cipher)!=1) || *cipher<0) {
 			    *cipher=-1;
@@ -764,6 +794,11 @@ get_options_stdin(char *ip, size_t iplen,
 				strncpy(op, val, oplen);
 			else
 				op[0] = 0;
+		} else if (!strcasecmp(name,"delay")) {
+			if (val)
+				strncpy(delay, val, delaylen);
+			else
+				delay[0] = 0;
 		}
 	}
 
@@ -795,8 +830,11 @@ printf("   -S <path>      Script to retrieve password (if required)\n");
 printf("   -l <login>     Username/Login (if required) to control power\n"
        "                  on IPMI device\n");
 printf("   -o <op>        Operation to perform.\n");
-printf("                  Valid operations: on, off, reboot, status, list or monitor\n");
+printf("                  Valid operations: on, off, reboot, status,\n");
+printf("                  diag, list or monitor\n");
 printf("   -t <timeout>   Timeout (sec) for IPMI operation (default %d)\n",DEFAULT_TIMEOUT);
+printf("   -T <timeout>   Wait X seconds after on/off operation\n");
+printf("   -f <timeout>   Wait X seconds before fencing is started\n");
 printf("   -C <cipher>    Ciphersuite to use (same as ipmitool -C parameter)\n");
 printf("   -M <method>    Method to fence (onoff or cycle (default %s)\n", DEFAULT_METHOD);
 printf("   -V             Print version and exit\n");
@@ -808,11 +846,13 @@ printf("   ipaddr=<#>            Same as -a\n");
 printf("   passwd=<pass>         Same as -p\n");
 printf("   passwd_script=<path>  Same as -S\n");
 printf("   lanplus               Same as -P\n");
-printf("   login=<login>         Same as -u\n");
+printf("   login=<login>         Same as -l\n");
 printf("   option=<op>           Same as -o\n");
 printf("   operation=<op>        Same as -o\n");
 printf("   action=<op>           Same as -o\n");
+printf("   delay=<seconds>       Same as -f\n");
 printf("   timeout=<timeout>     Same as -t\n");
+printf("   power_wait=<time>     Same as -T\n");
 printf("   cipher=<cipher>       Same as -C\n");
 printf("   method=<method>       Same as -M\n");
 printf("   verbose               Same as -v\n\n");
@@ -829,7 +869,10 @@ static void print_xml_metadata(char *pname) {
   printf("<longdesc>\n");
   printf("fence_ipmilan is an I/O Fencing agent which can be used with "
          "machines controlled by IPMI. This agent calls support software "
-         "using ipmitool (http://ipmitool.sf.net/).");
+         "using ipmitool (http://ipmitool.sf.net/).\n\n"
+         "To use fence_ipmilan with HP iLO 3 you have to enable lanplus "
+         "option (lanplus / -P) and increase wait after operation to "
+         "4 seconds (power_wait=4 / -T 4)");
   printf("</longdesc>\n");
   printf("%s\n","<parameters>");
 
@@ -856,6 +899,7 @@ static void print_xml_metadata(char *pname) {
   printf("\t<action name=\"%s\" />\n", "off");
   printf("\t<action name=\"%s\" />\n", "reboot");
   printf("\t<action name=\"%s\" />\n", "status");
+  printf("\t<action name=\"%s\" />\n", "diag");
   printf("\t<action name=\"%s\" />\n", "list");
   printf("\t<action name=\"%s\" />\n", "monitor");
   printf("\t<action name=\"%s\" />\n", "metadata");
@@ -874,12 +918,14 @@ main(int argc, char **argv)
 	char user[64];
 	char op[64];
 	char method[64];
+	char delay[64];
 	char pwd_script[PATH_MAX] = { 0, };
 	int lanplus=0;
 	int verbose=0;
 	char *pname = basename(argv[0]);
 	struct ipmi *i;
 	int timeout=DEFAULT_TIMEOUT;
+	int down_sleep=DEFAULT_POWER_WAIT;
 	int cipher=-1;
 	int print_final_status=1;
 	int translated_ret = -1;
@@ -890,12 +936,13 @@ main(int argc, char **argv)
 	memset(user, 0, sizeof(user));
 	memset(op, 0, sizeof(op));
 	memset(method, 0, sizeof(method));
+	memset(delay, 0, sizeof(delay));
 
 	if (argc > 1) {
 		/*
 		   Parse command line options if any were specified
 		 */
-		while ((opt = getopt(argc, argv, "A:a:i:l:p:S:Po:vV?hHt:C:M:")) != EOF) {
+		while ((opt = getopt(argc, argv, "A:a:i:l:p:S:Po:vV?hHt:T:C:M:f:")) != EOF) {
 			switch(opt) {
 			case 'A':
 				/* Auth type */
@@ -925,12 +972,22 @@ main(int argc, char **argv)
 				/* Operation */
 				strncpy(op, optarg, sizeof(op));
 				break;
+			case 'f':
+				/* Delay */
+				strncpy(delay, optarg, sizeof(delay));
+				break;
 			case 't':
 				/* Timeout */
 				if ((sscanf(optarg,"%d",&timeout)!=1) || timeout<1) {
 				    fail_exit("Timeout option expects positive number parameter");
 				}
 				break;
+			case 'T':
+				/* Shutdown Sleep Time */
+				if ((sscanf(optarg,"%d",&down_sleep)!=1) || down_sleep<1) {
+				    fail_exit("Shutdown time option expects positive number parameter");
+				}
+				break;
 			case 'C':
 				/* Ciphersuite */
 				if ((sscanf(optarg,"%d",&cipher)!=1) || cipher<0) {
@@ -946,7 +1003,7 @@ main(int argc, char **argv)
 				break;
 			case 'V':
         			printf("%s %s (built %s %s)\n", pname,
-				       RELEASE_VERSION,
+				       VERSION,
                				__DATE__, __TIME__);
         			printf("%s\n",
 				       REDHAT_COPYRIGHT);
@@ -965,7 +1022,9 @@ main(int argc, char **argv)
 					  pwd_script, sizeof(pwd_script),
 				      user, sizeof(user),
 				      op, sizeof(op), &lanplus, &verbose,&timeout,
-				      &cipher, method, sizeof(method)) != 0)
+				      &down_sleep,
+				      &cipher, method, sizeof(method),
+				      delay, sizeof(delay)) != 0)
 			return 1;
 	}
 
@@ -1013,9 +1072,9 @@ main(int argc, char **argv)
 	if (strcasecmp(op, "off") && strcasecmp(op, "on") &&
 	    strcasecmp(op, "status") && strcasecmp(op, "reboot") &&
 	    strcasecmp(op, "monitor") && strcasecmp(op, "list") &&
-	    strcasecmp(op, "metadata")) {
+	    strcasecmp(op, "metadata") && strcasecmp(op, "diag")) {
 		fail_exit("operation must be 'on', 'off', 'status', "
-			  "'reboot', 'list', 'monitor' or 'metadata'");
+			  "'reboot', 'diag', 'list', 'monitor' or 'metadata'");
 	}
 
 	if (strlen(authtype) &&
@@ -1036,8 +1095,15 @@ main(int argc, char **argv)
 		fail_exit("cycle method supports only 'reboot' operation (not 'on' or 'off').");
 	}
 
+	/* Delay fencing if requested */
+	if (delay) {
+		if (!strcasecmp(op, "reboot") || !strcasecmp(op, "off")) {
+			sleep(atoi(delay));
+		}
+	}
+
 	/* Ok, set up the IPMI struct */
-	i = ipmi_init(NULL, ip, authtype, user, passwd, lanplus, verbose, timeout, cipher);
+	i = ipmi_init(NULL, ip, authtype, user, passwd, lanplus, verbose, timeout, down_sleep, cipher);
 	if (!i)
 		fail_exit("Failed to initialize\n");
 
@@ -1106,6 +1172,12 @@ main(int argc, char **argv)
 	  ret=0;
 	  translated_ret = ERR_OK;
 	  print_final_status=0;
+	} else if (!strcasecmp(op, "diag")) {
+	  printf("Pulsing diagnostic interrupt to machine @ IPMI:%s...", ip);
+	  fflush(stdout);
+	  ret = ipmi_diag(i);
+	  translated_ret = (ret==0?ERR_ON_SUCCESSFUL:ERR_ON_FAIL);
+	  ret = 0;
 	}
 
 
@@ -1120,5 +1192,18 @@ metaout:
 	  else
 	    printf("Failed\n");
 	}
+	
+	if (!strcasecmp(op, "monitor"))
+          translated_ret = ret;
+        
+        if (!strcasecmp(op, "diag")) {
+          /** .. but when a system receive the DIAG signal , it 
+           ** switches to a kdump kernel, but the machine is always
+           ** "on" during the dump phase. It only become temporarily
+           ** "off" at the end of the dump just before rebooting.
+           **/
+          translated_ret = 0;
+        }
+
 	return translated_ret;
 }
