@@ -23,11 +23,11 @@ COMMAND_PROMPT_NEW="[PEXPECT]"
 # Start comunicating after login. Prepare good environment.
 def start_communication(conn, options):
 	conn.sendline ("PS1='"+COMMAND_PROMPT_NEW+"'")
-	res=conn.expect([pexpect.TIMEOUT, COMMAND_PROMPT_REG],SHELL_TIMEOUT)
+	res=conn.expect([pexpect.TIMEOUT, COMMAND_PROMPT_REG],int(options["-Y"]))
 	if res==0:
 		#CSH stuff
 		conn.sendline("set prompt='"+COMMAND_PROMPT_NEW+"'")
-		conn.log_expect(options, COMMAND_PROMPT_REG,SHELL_TIMEOUT)
+		conn.log_expect(options, COMMAND_PROMPT_REG,int(options["-Y"]))
 	
 
 def get_power_status(conn, options):
@@ -36,7 +36,7 @@ def get_power_status(conn, options):
 		
 		conn.sendline("ldm ls")
 		    
-		conn.log_expect(options,COMMAND_PROMPT_REG,SHELL_TIMEOUT)
+		conn.log_expect(options,COMMAND_PROMPT_REG,int(options["-Y"]))
 
 		result={}
 
@@ -73,7 +73,7 @@ def set_power_status(conn, options):
             	
 		conn.sendline(cmd_line)
 		    
-		conn.log_expect(options,COMMAND_PROMPT_REG,POWER_TIMEOUT)
+		conn.log_expect(options,COMMAND_PROMPT_REG,int(options["-g"]))
 		
 	except pexpect.EOF:
 		fail(EC_CONNECTION_LOST)
@@ -84,7 +84,8 @@ def main():
 	device_opt = [  "help", "version", "agent", "quiet", "verbose", "debug",
 			"action", "ipaddr", "login", "passwd", "passwd_script",
 			"secure",  "identity_file", "test" , "port", "cmd_prompt",
-			"separator", "inet4_only", "inet6_only", "ipport" ]
+			"separator", "inet4_only", "inet6_only", "ipport",
+			"power_timeout", "shell_timeout", "login_timeout", "power_wait" ]
 
 	atexit.register(atexit_handler)
 
@@ -98,13 +99,28 @@ def main():
 	if (not options.has_key("-c")):
 		options["-c"] = "\ $"
 	
-	show_docs(options)
+	docs = { }
+	docs["shortdesc"] = "Fence agent for Sun LDOM"
+	docs["longdesc"] = "fence_ldom is an I/O Fencing agent \
+which can be used with LDoms virtual machines. This agent works \
+so, that run ldm command on host machine. So ldm must be directly \
+runnable.\
+\n.P\n\
+Very useful parameter is -c (or cmd_prompt in stdin mode). This \
+must be set to something, what is displayed after successful login \
+to host machine. Default string is space on end of string (default \
+for root in bash). But (for example) csh use ], so in that case you \
+must use parameter -c with argument ]. Very similar situation is, \
+if you use bash and login to host machine with other user than \
+root. Than prompt is $, so again, you must use parameter -c."
+	docs["vendorurl"] = "http://www.sun.com"
+	show_docs(options, docs)
 
 	##
 	## Operate the fencing device
 	####
 	conn = fence_login(options)
-	fence_action(conn, options, set_power_status, get_power_status,get_power_status)
+	result = fence_action(conn, options, set_power_status, get_power_status,get_power_status)
 
 	##
 	## Logout from system
@@ -115,7 +131,9 @@ def main():
 	except exceptions.OSError:
 		pass
 	except pexpect.ExceptionPexpect:
-		pass	
+		pass
+
+	sys.exit(result)		
 
 if __name__ == "__main__":
 	main()
